from django.conf import settings
from django.views.generic import TemplateView


class LoginUrlTestTemplateView(TemplateView):
    def get_context_data(self, *args, **kwargs):
        context = super(LoginUrlTestTemplateView, self).get_context_data(*args, **kwargs)
        print(context)
        context['login_url'] = settings.LOGIN_URL
        return context


from rest_framework import serializers
from django.contrib.auth import get_user_model
from django.contrib.staticfiles.templatetags.staticfiles import static

from comments.models import Comment
User = get_user_model()

class UserPublicSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = [
            'username',
            'first_name',
            'last_name',
        ]

class CommentSerializer(serializers.ModelSerializer):
    user = UserPublicSerializer(read_only=True)
    image = serializers.SerializerMethodField(read_only=True)
    class Meta:
        model = Comment
        fields = ('id', 'url', 'user', 'content', 'timestamp','updated','image')

    def get_image(self, obj):
        img_  = static("comments/img/user.png")
        # if obj.user.profile.image:
            # img_ = obj.user.profile.image
        return img_   

class CommentUpdateSerializer(serializers.ModelSerializer):
    user = UserPublicSerializer(read_only=True)
    image = serializers.SerializerMethodField(read_only=True)
    class Meta:
        model = Comment
        fields = ('id', 'url', 'user', 'content', 'timestamp', 'updated', 'image')
        read_only_fields = ['url']
        
    def get_image(self, obj):
        img_  = static("comments/img/user.png")
        # if obj.user.profile.image:
            # img_ = obj.user.profile.image
        return img_


from django import template

register = template.Library()

from comments.models import Comment

@register.inclusion_tag('comments/comments-loader.html')
def load_comments(url):
    qs  = Comment.objects.filter(url=url)
    return {'comments':qs}


from django_elasticsearch_dsl import DocType, Index
from search.models import Post

posts = Index('posts')

@posts.doc_type
class PostDocument(DocType):
    class Meta:
        model = Post

        fields = [
            'title',
            'id',
            'slug',
            'image',
            'description',
        ]


"""calvineComments URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.11/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""
from django.conf.urls import url, include
from django.contrib import admin
from django.views.generic import TemplateView
from search import views as search_views 

from .views import LoginUrlTestTemplateView
urlpatterns = [
    url(r'^$', TemplateView.as_view(template_name="home.html")),
    url(r'^1234/$', LoginUrlTestTemplateView.as_view(template_name="home.html")),
    url(r'^admin/', admin.site.urls),
    url(r'^api/comments/', include('comments.api.urls')),
    # url('', include('django_prometheus.urls')),
    url('comments/', include('comments.urls')),
    url(r'^search/', search_views.search, name='search'),
]




# from django.conf import settings
# from django.conf.urls.static import static

# if settings.DEBUG:
#     urlpatterns += (static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT))
#     urlpatterns += (static(settings.STATIC_URL, document_root=settings.STATIC_ROOT))




from rest_framework import permissions

class IsOwnerOrReadOnly(permissions.BasePermission):
    """
    Object-level permission to only allow owners of an object to edit it.
    Assumes the model instance has an `owner` attribute.
    """
    def has_object_permission(self, request, view, obj):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True

        # Instance must have an attribute named `user`.
        return obj.user == request.user


from django import forms
from django.contrib.auth.models import User, Group
from django.core.exceptions import ValidationError

from .models import Post, CommentPost

class PostForm(forms.ModelForm):
    class Meta:
        model = Post
        fields = ('content', )
    def clean_content(self):
        data = self.cleaned_data.get('content')
        if len(data) <= 10:
            raise forms.ValidationError('Please enter at least 10 characters')
        return data

class CommentPostForm(forms.ModelForm):
    
    class Meta:
        model = CommentPost
        fields = ('title', 'text',)


# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-07-15 08:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('image', models.ImageField(upload_to='post_images')),
                ('body', models.TextField(blank=True, null=True)),
                ('order', models.IntegerField(blank=True, null=True)),
                ('slug', models.SlugField(blank=True, default='')),
            ],
        ),
    ]



from django.db import models

from django.conf import settings
from django.db import models
from django.utils import timezone

# comments via urls and users

class Comment(models.Model):
    user        = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True,on_delete=models.CASCADE)
    url         = models.URLField() 
    content     = models.TextField()
    #image       = models.ImageField()
    allow_annon = models.BooleanField(default=True)
    timestamp   = models.DateTimeField(auto_now_add=True)
    updated     = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.url


    @property
    def owner(self):
        return self.user


class Post(models.Model):
    content = models.TextField()

    def get_excerpt(self, char):
        return self.content[:char]


class CommentPost(models.Model):
    """
    model Post
    """
    author = models.ForeignKey('auth.User', on_delete=models.CASCADE)
    title = models.CharField(max_length=200)
    text = models.TextField()
    created_date = models.DateTimeField(
        default=timezone.now)
    published_date = models.DateTimeField(
        blank=True, null=True)

    def publish(self):
        self.published_date = timezone.now()
        self.save()

    def __str__(self):
        return self.title





from rest_framework.views import APIView
from rest_framework import generics
from rest_framework.mixins import DestroyModelMixin
from rest_framework.response import Response
from rest_framework import authentication, permissions

from comments.models import Comment
from .serializers import CommentSerializer, CommentUpdateSerializer

from .permissions import IsOwnerOrReadOnly

class CommentListAPIView(generics.ListAPIView):
    # queryset = Comment.objects.all()
    serializer_class = CommentSerializer
    # authentication_classes = []
    permission_classes = []

    def get_queryset(self, *args, **kwargs):
        url = self.request.GET.get("url")
        if url:
            return Comment.objects.filter(url=url)
        return Comment.objects.none()

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        response = Response(serializer.data)
        response.set_cookie('isUser', 'false')
        print(request.user)

        if request.user.is_authenticated():
           
            response.set_cookie('isUser', 'true')
            response.set_cookie('authUsername', str(request.user.username))
        return response

class CommentCreateAPIView(generics.CreateAPIView):
    
    queryset = Comment.objects.all()
    serializer_class = CommentSerializer

    def perform_create(self, serializer):
        if self.request.user.is_authenticated():
            serializer.save(user=self.request.user)

class CommentUpdateAPIView(DestroyModelMixin, generics.RetrieveUpdateAPIView):
    queryset = Comment.objects.all()
    serializer_class = CommentUpdateSerializer
    permission_classes = [IsOwnerOrReadOnly]

    def perform_update(self, serializer):
        if self.request.user.is_authenticated():
            serializer.save(user=self.request.user)

    def delete(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)


from django.conf.urls import url, include

from django.views.generic import TemplateView
from .views import CommentListAPIView, CommentCreateAPIView, CommentUpdateAPIView
# from .views import LoginUrlTestTemplateView
urlpatterns = [
    url(r'^$', CommentListAPIView.as_view(), name='list'),
    url(r'^create/$', CommentCreateAPIView.as_view(), name='create'),
    url(r'^(?P<pk>\d+)/$', CommentUpdateAPIView.as_view(), name='update'),
  
]


from django.contrib.auth.models import User
from django.test import TestCase
from django.utils import timezone

from .models import Post, CommentPost
from .forms import PostForm,CommentPostForm


class PostTestCase(TestCase):
    def setUp(self):
        self.user1 = User.objects.create_user(username="admin")
        CommentPost.objects.create(author=self.user1,
                            title="Test",
                            text="We are testing this",
                            created_date=timezone.now(),
                            published_date=timezone.now())

    def test_post_is_posted(self):
        """Posts are created"""
        post1 = CommentPost.objects.get(title="Test")
        self.assertEqual(post1.text, "We are testing this")

    def test_valid_form_data(self):
        form = CommentPostForm({
            'title': "Just testing",
            'text': "Repeated tests make the app foul-proof",
        })
        self.assertTrue(form.is_valid())
        post1 = form.save(commit=False)
        post1.author = self.user1
        post1.save()
        self.assertEqual(post1.title, "Just testing")
        self.assertEqual(post1.text, "Repeated tests make the app foul-proof")

    def test_blank_form_data(self):
        form = CommentPostForm({})
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors, {
            'title': ['This field is required.'],
            'text': ['This field is required.'],
        })



from django.conf.urls import url, include
from . import views

urlpatterns = [
    url('', views.post_list, name='post_list'),
    url('post/<int:pk>/', views.post_detail, name='post_detail'),
    url('post/new', views.post_new, name='post_new'),
    url('post/<int:pk>/edit/', views.post_edit, name='post_edit'),
]


from django.contrib import admin
from comments.models import *

from . import models

# Register your models here.
from .models import Comment, Post,CommentPost



# Register your models here.

class PostAdmin(admin.ModelAdmin):
    model = models.Post
    list_display = ('excerpt',)

    def excerpt(self, obj):
        return obj.get_excerpt(5)


admin.site.register(Comment)

admin.site.register(models.CommentPost)
admin.site.register(models.Post,PostAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-07-13 13:26
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('comments', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CommentPost',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200)),
                ('text', models.TextField()),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('published_date', models.DateTimeField(blank=True, null=True)),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField()),
            ],
        ),
    ]



from django.shortcuts import render
from django.views.generic import TemplateView, FormView
from django.utils.decorators import method_decorator
from django.shortcuts import render, get_object_or_404, redirect
from django.utils import timezone
from .models import Post, CommentPost
from .forms import PostForm, CommentPostForm

# Create your views here.


class HomeView(TemplateView):
    template_name = 'comments/home.html'




def post_list(request):
    posts = CommentPost.objects.filter(published_date__lte=timezone.now()).order_by(
        'published_date')
    return render(request, 'blog/post_list.html', {'posts': posts})


def post_detail(request, pk):
    post = get_object_or_404(Post, pk=pk)
    return render(request, 'blog/post_detail.html', {'post': post})


def post_new(request):
    if request.method == "POST":
        form = CommentPostForm(request.POST)
        if form.is_valid():
            post = form.save(commit=False)
            post.author = request.user
            post.published_date = timezone.now()
            post.save()
            return redirect('post_detail', pk=post.pk)
    else:
        form = CommentPostForm()
    return render(request, 'blog/post_edit.html', {'form': form})


def post_edit(request, pk):
    post = get_object_or_404(Post, pk=pk)
    if request.method == "POST":
        form = CommentPostForm(request.POST, instance=post)
        if form.is_valid():
            post = form.save(commit=False)
            post.author = request.user
            post.published_date = timezone.now()
            post.save()
            return redirect('post_detail', pk=post.pk)
    else:
        form = CommentPostForm(instance=post)
    return render(request, 'blog/post_edit.html', {'form': form})


# class AdminView(TemplateView):
#     template_name = 'employee/admin.html'

#     @method_decorator(login_required)
#     def dispatch(self, request, *args, **kwargs):
#         return super(AdminView, self).dispatch(request,*args,*kwargs)

    
# class PostUpdateView(UpdateView):
#     model = models.Post
#     form_class = forms.PostForm
#     success_url = '/'

#     def post(self, request, *args, **kwargs):
#         if getattr(request.user, 'first_name', None) == 'Martin':
#             raise Http404()
#         return super(PostUpdateView, self).post(request, *args, **kwargs)



from django.shortcuts import render

# Create your views here.


from search.documents import PostDocument

def search(request):

    q = request.GET.get('q')

    if q:
        posts = PostDocument.search().query("match", title=q)
    else:
        posts = ''

    return render(request, 'search/search.html', {'posts': posts})


from django.db import models

# Create your models here.



from django.utils.text import slugify

class Post(models.Model):

    title = models.CharField(max_length=255, blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    image = models.ImageField(upload_to="post_images")
    body = models.TextField(blank=True, null=True)
    order = models.IntegerField(blank=True, null=True)

    slug = models.SlugField(default='', blank=True)

    def save(self):
        self.slug = slugify(self.title)
        super(Post, self).save()

    def __str__(self):
        return '%s' % self.title
